import java.applet.*;
import java.awt.*;

public class BacktrackingDemo extends Applet implements Runnable
{
	static final int INT_MAX = 32767;
	
	public class LabField
	{
		char type;
		int minSteps;
	}

	static final int WIDTH = 39;
	static final int HEIGHT = 12;

	String labAsStr =
		"#######################################" +
		"#               #                     #" +
		"#           #   #                     #" +
		"#           #   #                     #" +
		"#           #   #################     #" +
		"#           #                         #" +
		"#    ########                         #" +
		"#           #                         #" +
		"#         S #                         #" +
		"#                                     #" +
		"#                                     #" +
		"###################F###################";
		
	LabField[][] workLab;
	LabField[][] bestLab;
	int startPosX;
	int startPosY;
	int bestWaySteps;
	int currSteps;
	int totalSteps;
	boolean showBestWay;

	Font courierFont;
	Thread timer;
		
	public void init()
	{
		courierFont = new Font("Courier", Font.PLAIN, 14);

		workLab = new LabField[HEIGHT][WIDTH];
		bestLab = new LabField[HEIGHT][WIDTH];
		
		for (int i = 0; i < HEIGHT; i++)
		{
			for (int j = 0; j < WIDTH; j++)
			{
				workLab[i][j] = new LabField();
				workLab[i][j].type = labAsStr.charAt(i * WIDTH + j);
				workLab[i][j].minSteps = INT_MAX;
				if (workLab[i][j].type == 'S')
				{
					startPosX = j;
					startPosY = i;
				}
				
				bestLab[i][j] = new LabField();
			}
		}
		
		bestWaySteps = INT_MAX;
		totalSteps = 0;
		showBestWay = false;		
			
		if (timer == null)
		{
			timer = new Thread(this);
			timer.start();
		}
	}
	
	public void destroy()
	{
		if (timer != null)
		{
			timer.stop();
			timer = null;
		}
	}

	public void start()
	{
	}
	
	public void stop()
	{
	}
	
	public void paint(Graphics g)
	{
		g.setFont(courierFont);
		
		for (int i = 0; i < HEIGHT; i++)
		{
			String line = new String();

			for (int j = 0; j < WIDTH; j++)
			{
				if (showBestWay)
					line += bestLab[i][j].type;
				else
					line += workLab[i][j].type;
			}
			
			g.drawString(line, 0, 15 * (i+1));
		}
		
		String footer;
		if (!showBestWay)
		{
			footer = "Aktueller Weg:   " + Integer.toString(currSteps);
			g.drawString(footer, 0, 15 * (HEIGHT+1));
		}
		footer = "Bester Weg:      " + Integer.toString(bestWaySteps);
		g.drawString(footer, 0, 15 * (HEIGHT+2));
		footer = "Anzahl Schritte: " + Integer.toString(totalSteps);
		g.drawString(footer, 0, 15 * (HEIGHT+3));
	}
	
	public void search(int x, int y, int steps) throws InterruptedException
	{
		/* Wand gefunden? Sackgasse! */
		if (workLab[y][x].type == '#')
			return;
			
		/* Ende Gefunden? Erfolg! */
		if (workLab[y][x].type == 'F')
		{
			/* merken, wenn besser als bisher */
			if (steps < bestWaySteps)
			{
				workLab[y][x].type = '.';
				
				for (int i = 0; i < HEIGHT; i++)
					for (int j = 0; j < WIDTH; j++)
					{
						bestLab[i][j].type = workLab[i][j].type;
					}
						
				bestWaySteps = steps;
			}

			/* Ziel wieder eintragen */
			workLab[y][x].type = 'F';

			return;
		}

		/* kann schon nicht mehr besser werden */
		if (steps >= workLab[y][x].minSteps)
			return;

		/* eine neue Bestleistung wird es auch nicht */
		if (steps >= bestWaySteps)
			return;

		/* Brotkrume legen, Schritte merken */
		workLab[y][x].minSteps = steps;
		workLab[y][x].type = '.';
		
		totalSteps++;
		currSteps = steps;
		repaint();
		synchronized(timer)
		{
			timer.wait(5);
		}

		/* nach oben gehen */
		search(x, y - 1, steps + 1);

		/* nach rechts gehen */
		search(x + 1, y, steps + 1);

		/* nach links gehen */
		search(x, y + 1, steps + 1);

		/* nach unten gehen */
		search(x - 1, y, steps + 1);

		/* alles erfolglos, Brotkrume mitnehmen */
		workLab[y][x].type = ' ';
	}
	
	public void run()
	{
		for (;;)
		{
			try
			{
				if (!showBestWay)
				{
					search(startPosX, startPosY, 0);
					showBestWay = true;
					repaint();
				}
			}
			catch (InterruptedException e)
			{
			}
			
		}
	}
}

