/*
 * Copyright (c) 2005
 *    Felix Opatz <felix@zotteljedi.de>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY FELIX OPATZ ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL FELIX OPATZ BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#define NODES	12
#define FINISH	11
#define EDGES	17
#define ILLEGAL	(-1)

int best_sum = 9999;
int last_depth;

void fill_matrix(int matrix[NODES][NODES])
{
	int i, j;

	for (i = 0; i < NODES; i++)
		for (j = 0; j < NODES; j++)
			matrix[i][j] = ILLEGAL;

	matrix[0][1] = 6;
	matrix[0][2] = 4;

	matrix[1][3] = 5;
	matrix[1][5] = 2;

	matrix[2][4] = 2;

	matrix[3][4] = 4;
	matrix[3][5] = 2;
	matrix[3][6] = 3;

	matrix[4][7] = 2;
	matrix[4][8] = 3;

	matrix[5][6] = 5;

	matrix[6][10] = 4;

	matrix[7][9] = 3;
	matrix[7][10] = 5;

	matrix[8][9] = 7;

	matrix[9][11] = 6;

	matrix[10][11] = 2;
}

void mirror_matrix(int matrix[NODES][NODES])
{
	int i, j;

	for (i = 0; i < NODES; i++)
		for (j = 0; j < NODES; j++)
			matrix[j][i] = matrix[i][j];
}

void print_way(int way[EDGES + 1], int depth, int sum)
{
	int i;

	printf("%i: ", sum);

	for (i = 0; i <= depth; i++)
		printf("%s%i", (i ? " -> " : ""), way[i]);

	putchar('\n');
}

int find_way(int matrix[NODES][NODES], int node, int way[EDGES + 1],
	int depth, int sum, int best_way[EDGES + 1])
{
	int i, ret = 0, old;

	assert(depth <= EDGES);

	way[depth] = node;

	if (sum > best_sum)
		return 1;

	if (node == FINISH)
	{
		best_sum = sum;
		last_depth = depth;
		for (i = 0; i <= depth; i++)
			best_way[i] = way[i];
		return 1;
	}

	for (i = 0; i < NODES; i++)
	{
		if (matrix[node][i] != ILLEGAL)
		{
			old = matrix[node][i];
			matrix[node][i] = ILLEGAL;
			matrix[i][node] = ILLEGAL;
			if (find_way(matrix, i, way, depth + 1, sum + old, best_way))
				ret = 1;
			matrix[node][i] = old;
			matrix[i][node] = old;
		}
	}

	return ret;
}

int main(void)
{
	int matrix[NODES][NODES];
	int way[EDGES + 1];
	int best_way[EDGES + 1];

	fill_matrix(matrix);
	mirror_matrix(matrix);

	if (!find_way(matrix, 0, way, 0, 0, best_way))
		printf("there is no way\n");
	else
		print_way(best_way, last_depth, best_sum);

	return 0;
}
