/*
 * Copyright (c) 2005
 *    Felix Opatz <felix@zotteljedi.de>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY FELIX OPATZ ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL FELIX OPATZ BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

#define MAX_LINES	100
#define	MAX_COLS	100

char lab[MAX_LINES][MAX_COLS];

int x_start, y_start, max_lines, max_cols;

int load_lab(const char *path)
{
	FILE *file;
	char line[MAX_COLS + 1];
	int i, j;

	if (!(file = fopen(path, "r")))
	{
		perror("fopen() failed");
		return 1;
	}

	for (j = 0; fgets(line, sizeof(line), file) && j < MAX_LINES; j++)
	{
		for (i = 0; i < MAX_COLS && line[i] && line[i] != '\n'; i++)
		{
			lab[j][i] = line[i];
			if (line[i] == 'S')
			{
				x_start = i;
				y_start = j;
			}
		}

	}
	max_cols = i - 1;
	max_lines = j - 1;

	fclose(file);

	return 0;
}

int print_lab(void)
{
	int i, j;

	for (j = 0; j < MAX_LINES && lab[j][0]; j++)
	{
		for (i = 0; i < MAX_COLS && lab[j][i]; i++)
			putchar(lab[j][i]);
		putchar('\n');
	}

	return 0;
}

int search_lab(int x_pos, int y_pos)
{
	/* Wand gefunden? Sackgasse! */
	if (lab[y_pos][x_pos] == '#')
		return 0;

	/* Ende Gefunden? Erfolg! */
	if (lab[y_pos][x_pos] == 'F')
		return 1;

	/* Schleife? Zruck! */
	if (lab[y_pos][x_pos] == '.')
		return 0;

	/* Brotkrume hinlegen */
	lab[y_pos][x_pos] = '.';

	/* nach oben gehen */
	if (search_lab(x_pos, y_pos - 1))
		return 1;

	/* nach rechts gehen */
	if (search_lab(x_pos + 1, y_pos))
		return 1;

	/* nach unten gehen */
	if (search_lab(x_pos, y_pos + 1))
		return 1;

	/* nach links gehen */
	if (search_lab(x_pos - 1, y_pos))
		return 1;

	/* alles erfolglos, Brotkrume mitnehmen */
	lab[y_pos][x_pos] = ' ';

	return 0;
}

int main(int argc, char *argv[])
{
	if (argc < 2)
	{
		fprintf(stderr, "usage: %s <lab-file>\n", argv[0]);
		return 1;
	}

	if (load_lab(argv[1]))
		return 2;

	search_lab(x_start, y_start);
	print_lab();

	return 0;
}

