/*
 * Copyright (c) 2005
 *    Felix Opatz <felix@zotteljedi.de>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY FELIX OPATZ ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL FELIX OPATZ BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

#define MAX_LINES	100
#define	MAX_COLS	100

struct {
	char cell_type;
	int	min_steps;
} lab[MAX_LINES][MAX_COLS], best_lab[MAX_LINES][MAX_COLS];

int x_start, y_start, max_lines, max_cols;
int best_way = INT_MAX;

int load_lab(const char *path)
{
	FILE *file;
	char line[MAX_COLS + 1];
	int i, j;

	if (!(file = fopen(path, "r")))
	{
		perror("fopen() failed");
		return 1;
	}

	for (j = 0; fgets(line, sizeof(line), file) && j < MAX_LINES; j++)
	{
		for (i = 0; i < MAX_COLS && line[i] && line[i] != '\n'; i++)
		{
			lab[j][i].cell_type = line[i];
			lab[j][i].min_steps = INT_MAX;
			if (line[i] == 'S')
			{
				x_start = i;
				y_start = j;
			}
		}

	}
	max_cols = i - 1;
	max_lines = j - 1;

	fclose(file);

	return 0;
}

int print_lab(void)
{
	int i, j;

	for (j = 0; j < MAX_LINES && lab[j][0].cell_type; j++)
	{
		for (i = 0; i < MAX_COLS && lab[j][i].cell_type; i++)
			putchar(lab[j][i].cell_type);
		putchar('\n');
	}

	return 0;
}

void search_lab(int x_pos, int y_pos, int steps)
{
	/* Wand gefunden? Sackgasse! */
	if (lab[y_pos][x_pos].cell_type == '#')
		return;

	/* Ende Gefunden? Erfolg! */
	if (lab[y_pos][x_pos].cell_type == 'F')
	{
		/* merken, wenn besser als bisher */
		if (steps < best_way)
		{
			lab[y_pos][x_pos].cell_type = '.';
			memcpy(best_lab, lab, sizeof(lab));
			best_way = steps;
		}

		/* da habe ich ewig dran gesucht... */
		lab[y_pos][x_pos].cell_type = 'F';

		return;
	}

	/* kann schon nicht mehr besser werden */
	if (steps >= lab[y_pos][x_pos].min_steps)
		return;

	/* eine neue Bestleistung wird es auch nicht */
	if (steps >= best_way)
		return;

	/* Brotkrume legen, Schritte merken */
	lab[y_pos][x_pos].min_steps = steps;
	lab[y_pos][x_pos].cell_type = '.';

	/* nach oben gehen */
	search_lab(x_pos, y_pos - 1, steps + 1);

	/* nach rechts gehen */
	search_lab(x_pos + 1, y_pos, steps + 1);

	/* nach unten gehen */
	search_lab(x_pos, y_pos + 1, steps + 1);

	/* nach links gehen */
	search_lab(x_pos - 1, y_pos, steps + 1);

	/* alles erfolglos, Brotkrume mitnehmen */
	lab[y_pos][x_pos].cell_type = ' ';
}

int main(int argc, char *argv[])
{
	if (argc < 2)
	{
		fprintf(stderr, "usage: %s <lab-file>\n", argv[0]);
		return 1;
	}

	if (load_lab(argv[1]))
		return 2;

	search_lab(x_start, y_start, 0);

	memcpy(lab, best_lab, sizeof(lab));
	print_lab();

	printf("Der Weg hat %i Schritte\n", best_way);

	return 0;
}


