/*
 * Copyright (c) 2005
 *    Felix Opatz <felix@zotteljedi.de>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY FELIX OPATZ ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL FELIX OPATZ BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#define NODES	5
#define EDGES	8

void fill_matrix(int matrix[NODES][NODES])
{
	int i, j;

	for (i = 0; i < NODES; i++)
		for (j = 0; j < NODES; j++)
			matrix[i][j] = 0;

	matrix[0][1] = 1;
	matrix[0][3] = 1;
	matrix[0][4] = 1;

	matrix[1][2] = 1;
	matrix[1][3] = 1;
	matrix[1][4] = 1;

	matrix[2][3] = 1;

	matrix[3][4] = 1;
}

void mirror_matrix(int matrix[NODES][NODES])
{
	int i, j;

	for (i = 0; i < NODES; i++)
		for (j = 0; j < NODES; j++)
			matrix[j][i] = matrix[i][j];
}

void print_way(int way[EDGES + 1], int depth)
{
	int i;

	for (i = 0; i <= depth; i++)
		printf("%s%i", (i ? "-> " : ""), way[i]);

	putchar('\n');
}

int find_way(int matrix[NODES][NODES], int node, int way[EDGES + 1], int depth)
{
	int i, ret = 0;

	assert(depth <= EDGES);

	way[depth] = node;

	if (EDGES == depth)
	{
		print_way(way, depth);
		return 1;
	}

	for (i = 0; i < NODES; i++)
	{
		if (matrix[node][i])
		{
			matrix[node][i] = 0;
			matrix[i][node] = 0;
			if (find_way(matrix, i, way, depth + 1))
				ret = 1;
			matrix[node][i] = 1;
			matrix[i][node] = 1;
		}
	}

	return ret;
}

int main(void)
{
	int matrix[NODES][NODES];
	int way[EDGES + 1];

	fill_matrix(matrix);
	mirror_matrix(matrix);

	if (!find_way(matrix, 0, way, 0))
		printf("there is no way\n");

	return 0;
}
